unit eng.box2d;

interface
  uses UPhysics2DTypes, UPhysics2D, OpenGL;

type
  Tb2Vec2 = TVector2;

function b2vec2(x, y: single): Tb2Vec2;

type
  TContactState = (e_contactAdded, e_contactPersisted, e_contactRemoved);
  TContactPoint = record
     b2point: Tb2ContactPoint;
     state: TContactState;
  end;

  TContactListener = class(Tb2ContactListener)
  private
    function CollideWithFlipper(point: Tb2ContactPoint): boolean;
  public
    m_points: array[0..2048-1] of TContactPoint;
    m_pointCount: integer;
    coll_flip: boolean;

    p: Tb2ContactPoint;
    lh: single;

    procedure Add(var point: Tb2ContactPoint); override;
    procedure Persist(var point: Tb2ContactPoint); override;
    procedure Remove(var point: Tb2ContactPoint); override;
    procedure Result(var point: Tb2ContactResult); override;
  end;

  TDebugDraw = class(Tb2DebugDraw)
    procedure DrawPolygon(const vertices: Tb2PolyVertices; vertexCount: Int32; const color: RGBA); override;
    procedure DrawPolygon4(const vertices: TVectorArray4; vertexCount: Int32; const color: RGBA); override;
    procedure DrawSolidPolygon(const vertices: Tb2PolyVertices; vertexCount: Int32; const color: RGBA); override;
    procedure DrawCircle(const center: TVector2; radius: Float; const color: RGBA); override;
    procedure DrawSolidCircle(const center, axis: TVector2; radius: Float; const color: RGBA); override;
    procedure DrawSegment(const p1, p2: TVector2; const color: RGBA); override;
    procedure DrawXForm(const xf: Tb2XForm); override;
  end;

var
  correct: Tb2vec2;

implementation
  uses pin.ball, pin.map, eng.utils, pin.flipper, eng.particles, pin.game;

function b2vec2(x, y: single): Tb2Vec2;
begin
  Result.x := x;
  Result.y := y;
end;

function FindGroup(id: integer): integer;
var
  i, j: integer;
begin
  Result := -1;
  for i := 0 to map.groups_count - 1 do
    for j := 0 to System.Length(map.groups[i].elements) - 1 do
      if map.groups[i].elements[j].id = id then
      begin
        Result := i;
        break;
      end;
end;

{ TContactListener }

procedure TContactListener.Add(var point: Tb2ContactPoint);
procedure CollideWithCylinders(point: Tb2ContactPoint);
var
  shape1, shape2: Tb2Shape;
  i: integer;
  group: integer;
  mulx: integer;
begin
  for i := 0 to 1 do
  begin
    case i of
      0: begin shape1 := point.shape1; shape2 := point.shape2; end;
      1: begin shape1 := point.shape2; shape2 := point.shape1; end;
    end;
    if (shape2.m_body = ball.body) and (shape2 = ball.sh_circle) and (shape1.IsSensor)
      and (TMapObjectData(shape1.m_body.UserData^).type_ = mot_Cylinder) then
    begin
      ball.health := ball.health - Length(ball.body.GetLinearVelocity)/Length(ball.max_velocity) * TMapObjectData(shape1.m_body.UserData^).dam;

      ball.body.SetLinearVelocity(b2vec2(0, 0));
      ball.body.ApplyImpulse(b2vec2(point.normal.x * shape1.m_restitution, point.normal.y * shape1.m_restitution), point.position);

      // Highscores
      group := FindGroup(TMapObjectData(shape1.m_body.UserData^).id);
      if group > -1 then
      begin
        mulx := map.groups[group].CountLikeCylinders(TMapObjectData(shape1.m_body.UserData^).id);
        inc(highscores, map.objects[TMapObjectData(shape1.m_body.UserData^).id].cost * mulx);
      end;

      inc(TMapObjectCylinder(map.objects[TMapObjectData(shape1.m_body.UserData^).id]).color);
      if (TMapObjectCylinder(map.objects[TMapObjectData(shape1.m_body.UserData^).id]).color > 3) then
        TMapObjectCylinder(map.objects[TMapObjectData(shape1.m_body.UserData^).id]).color := 0;

    end;
  end;
end;
procedure CollideWithHuyovins(point: Tb2ContactPoint);
var
  shape1, shape2: Tb2Shape;
  i: integer;
  group, mulx: integer;
begin
  for i := 0 to 1 do
  begin
    case i of
      0: begin shape1 := point.shape1; shape2 := point.shape2; end;
      1: begin shape1 := point.shape2; shape2 := point.shape1; end;
    end;
    if (shape2.m_body = ball.body) and (shape2 = ball.sh_circle) and (shape1.IsSensor)
      and (TMapObjectData(shape1.m_body.UserData^).type_ = mot_Huinya) then
    begin
      ball.health := ball.health - Length(ball.body.GetLinearVelocity)/Length(ball.max_velocity) * TMapObjectData(shape1.m_body.UserData^).dam;

      ball.body.SetLinearVelocity(b2vec2(0, 0));
      ball.body.ApplyImpulse(b2vec2(point.normal.x * shape1.m_restitution, point.normal.y * shape1.m_restitution), point.position);

      // Highscores
      group := FindGroup(TMapObjectData(shape1.m_body.UserData^).id);
      if group > -1 then
      begin
        mulx := map.groups[group].CountLikeHuini;
        inc(highscores, TMapObjectHuinya(map.objects[TMapObjectData(shape1.m_body.UserData^).id]).cost * mulx);
      end;
          
    end;
  end;
end;
procedure CollideWithSpeeders(point: Tb2ContactPoint);
var
  shape1, shape2: Tb2Shape;
  i: integer;
  dir: single;
begin
  for i := 0 to 1 do
  begin
    case i of
      0: begin shape1 := point.shape1; shape2 := point.shape2; end;
      1: begin shape1 := point.shape2; shape2 := point.shape1; end;
    end;
    if (shape2.m_body = ball.body) and (shape2 = ball.sh_circle) and (shape1.IsSensor)
      and (TMapObjectData(shape1.m_body.UserData^).type_ = mot_Speeder)
       and (TMapObjectSpeeder(map.objects[TMapObjectData(shape1.m_body.UserData^).id]).active) then
    begin
      dir := TMapObjectSpeeder(map.objects[TMapObjectData(shape1.m_body.UserData^).id]).direction;
      ball.v := b2vec2(cos(DegToRad(dir)) * 10, sin(DegToRad(dir)) * 10);
      ball.body.ApplyImpulse(ball.v, point.position);
    end;

  end;
end;
procedure CollideWithKicks(point: Tb2ContactPoint);
var
  shape1, shape2: Tb2Shape;
  i: integer;
begin
  for i := 0 to 1 do
  begin
    case i of
      0: begin shape1 := point.shape1; shape2 := point.shape2; end;
      1: begin shape1 := point.shape2; shape2 := point.shape1; end;
    end;
    if (shape2.m_body = ball.body) and (shape2 = ball.sh_circle) and (shape1.IsSensor)
      and (TMapObjectData(shape1.m_body.UserData^).type_ = mot_Kick) then
    begin
      ball.health := ball.health - Length(ball.body.GetLinearVelocity)/Length(ball.max_velocity) * TMapObjectData(shape1.m_body.UserData^).dam;

      ball.body.SetLinearVelocity(b2vec2(0, 0));
      ball.body.ApplyImpulse(b2vec2(point.normal.x * shape1.m_restitution, point.normal.y * shape1.m_restitution), point.position);

      inc(highscores, map.objects[TMapObjectData(shape1.m_body.UserData^).id].cost);
    end;
  end;
end;
var
  i, j: integer;
  vx, vy, ax, ay: integer;
  shape: Tb2Shape;
begin
   if m_pointCount = 2047 then
      Exit;

   m_points[m_pointCount].b2point := point;
   m_points[m_pointCount].state := e_contactAdded;
   Inc(m_pointCount);

   lh := ball.health;
   
   CollideWithCylinders(point);
   CollideWithHuyovins(point); 
   CollideWithFlipper(point);
   CollideWithKicks(point);

   CollideWithSpeeders(point);

   //pman.Add(point.position.x, point.position.y, point.normal.x*0.1, point.normal.y*0.1, 0.0, -0.005, 1.0);
  for j := 0 to 1 do
  begin
    case j of
      0: begin shape := point.shape1;end;
      1: begin shape := point.shape2;end;
    end;
   if shape.m_body.UserData <> nil then
   case (TMapObjectData(shape.m_body.UserData^).type_) of
     mot_Cylinder, mot_Kick, mot_Huinya, mot_Flip:
     begin
       for i := 0 to Round(abs(lh-ball.health)) do
       begin
         vx := random(100);
         vy := random(100);
         ax := random(100);
         ay := random(100);
         pman.Add(point.position.x, point.position.y, point.normal.x*0.1+vx/1000, point.normal.y*0.1+vy/1000, 0.0+ax/10000, -0.005+ay/10000, 1.0); 
       end;
     end;
     //mot_Null, mot_wall: pman.Add(point.position.x, point.position.y, point.normal.x*0.1, point.normal.y*0.1, 0.0, -0.005, 1.0);
   end;
  end;
   //mot_Cylinder, mot_Hole, mot_Trigger, mot_Huinya,
     //               mot_Flip, mot_Speeder, mot_Wall, mot_Kick
  
end;

function TContactListener.CollideWithFlipper(point: Tb2ContactPoint): boolean;
var
  shape1, shape2: Tb2Shape;
  i: integer;
begin
  Result := false;
  for i := 0 to 1 do
  begin
    case i of
      0: begin shape1 := point.shape1; shape2 := point.shape2; end;
      1: begin shape1 := point.shape2; shape2 := point.shape1; end;
    end;
    if (shape2.m_body = ball.body) and (shape2 = ball.sh_circle) and (shape1.m_body.IsDynamic)
      and (TMapObjectData(shape1.m_body.UserData^).type_ = mot_Flip) then
    begin 
      Result := true;
      {s := flipper[0].super;
      if s then
      begin
        ball.body.SetLinearVelocity(b2vec2(0, 0));
        ball.body.ApplyImpulse(b2vec2(point.normal.x * 5000, point.normal.y * 5000), point.position);
      end;  }
      if not coll_flip then
      begin
        ball.health := ball.health - Length(ball.body.GetLinearVelocity)/Length(ball.max_velocity) * 5;
        coll_flip := true;
        p := point;
        p.restitution := shape1.m_restitution;
      end;
    end;
  end;
end;

procedure TContactListener.Persist(var point: Tb2ContactPoint);
begin
   if m_pointCount = 2047 then
      Exit;

   m_points[m_pointCount].b2point := point;
   m_points[m_pointCount].state := e_contactPersisted;
   Inc(m_pointCount);
end;

procedure TContactListener.Remove(var point: Tb2ContactPoint);
begin
   if m_pointCount = 2047 then
      Exit;




   m_points[m_pointCount].b2point := point;
   m_points[m_pointCount].state := e_contactRemoved;
   Inc(m_pointCount);
end;

procedure TContactListener.Result(var point: Tb2ContactResult);
begin
  inherited;
end;

{ TDebugDraw }

procedure TDebugDraw.DrawCircle(const center: TVector2; radius: Float;
  const color: RGBA);
var
  i: integer;
begin
  glPushMatrix;
    glColor4fv(@color);
    glTranslatef(center.x, center.y, 0);
    glBegin(GL_POINTS);
    for i := 0 to 360 do
    begin
      glVertex2f(cos(i) * radius, sin(i) * radius);
    end;
    glEnd;
  glPopMatrix;
end;

procedure TDebugDraw.DrawPolygon(const vertices: Tb2PolyVertices;
  vertexCount: Int32; const color: RGBA);
var
  i: integer;
begin
  glPushMatrix;
    glColor4fv(@color);
    glBegin(GL_LINE_LOOP);
    for i := 0 to vertexCount - 1 do
    begin
      glVertex2f(vertices[i].x, vertices[i].y);
    end;
    glEnd;
  glPopMatrix;
end;

procedure TDebugDraw.DrawPolygon4(const vertices: TVectorArray4;
  vertexCount: Int32; const color: RGBA);
var
  i: integer;
begin
  glPushMatrix;
    glColor4fv(@color);
    glBegin(GL_LINE_LOOP);
    for i := 0 to 3 do
    begin
      glVertex2f(vertices[i].x, vertices[i].y);
    end;
    glEnd;
  glPopMatrix;
end;

procedure TDebugDraw.DrawSegment(const p1, p2: TVector2; const color: RGBA);
begin
  glPushMatrix;
    glColor4fv(@color);
    glBegin(GL_LINES);
      glVertex2f(p1.x, p1.y);
      glVertex2f(p2.x, p2.y);
    glEnd;
  glPopMatrix;
end;

procedure TDebugDraw.DrawSolidCircle(const center, axis: TVector2;
  radius: Float; const color: RGBA);
begin
  DrawCircle(center, radius, color);//!!!
end;

procedure TDebugDraw.DrawSolidPolygon(const vertices: Tb2PolyVertices;
  vertexCount: Int32; const color: RGBA);
begin
  DrawPolygon(vertices, vertexCount, color);//!!!
end;

procedure TDebugDraw.DrawXForm(const xf: Tb2XForm);
const
   k_axisScale = 0.4;
   clRed: rgba = (1.0, 0.0, 0.0, 1.0);
   clGreen: rgba = (0.0, 1.0, 0.0, 1.0);
var
   p2: TVector2;
begin
   with xf do
   begin
      p2.x := position.x + k_axisScale * R.col1.x;
      p2.y := position.y + k_axisScale * R.col1.y;
      glPushMatrix;
        glColor4fv(@clRed);
        glBegin(GL_LINES);
          glVertex2f(position.x, position.y);
          glVertex2f(p2.x, p2.y);
        glEnd;
      glPopMatrix;

      p2.x := position.x + k_axisScale * R.col2.x;
      p2.y := position.y + k_axisScale * R.col2.y;
      glPushMatrix;
        glColor4fv(@clGreen);
        glBegin(GL_LINES);
          glVertex2f(position.x, position.y);
          glVertex2f(p2.x, p2.y);
        glEnd;
      glPopMatrix;
   end;
end;

end.
