unit eng.forms;

interface
  uses OpenGl, eXgine, eng.box2d, UPhysics2D, UPhysics2DTypes;

type
  T2DBox = record
    position: Tb2Vec2;
    angle: single;
    halfsize: Tb2Vec2;
    procedure Draw;
  end;

  T2DCircle = record
    position: Tb2Vec2;
    radius: single;
    procedure Draw;
  end;

  T2DTriangle = record
    verticies: array[0..2] of TVector2D;
    procedure params(verticies: array of TVector2D);
    procedure Draw;
  end;

  T2DTriangleBox = record
    triangles: array[0..1] of T2DTriangle;
    position: Tb2Vec2;
    size: Tb2Vec2;
    procedure params(position: Tb2Vec2; Size: Tb2Vec2);
    procedure Draw;
  end;

  T2DSuperBox = record
    boxes: array of T2DTriangleBox;
    position: Tb2Vec2;
    size: Tb2Vec2;
    procedure params(position, size: Tb2Vec2; boxesc: integer = 1);
    procedure Draw;
  end;

  procedure DrawPolygon(shape: Tb2Shape; const xf: Tb2XForm);
  procedure DrawCircle(shape: Tb2Shape; const xf: Tb2XForm);

  procedure DrawLine(x, y, l, a: double);

  function vecXtoB2(vec: TVector2D; scale: double = 1.0): Tb2vec2;

implementation

function vecXtoB2;
begin
  Result.x := vec.X * scale;
  Result.y := vec.Y * scale;
end;

{ T2DBox }

procedure T2DBox.Draw;
begin
  glPushMatrix;
    glTranslatef(position.x, position.y, 0);
    glRotatef(angle{ * 180 / Pi}, 0, 0, 1);
    glBegin(GL_QUADS);
      glTexCoord2f(0, 0); glVertex2f(- halfsize.x, - halfsize.y);
      glTexCoord2f(1, 0); glVertex2f(+ halfsize.x, - halfsize.y);
      glTexCoord2f(1, 1); glVertex2f(+ halfsize.x, + halfsize.y);
      glTexCoord2f(0, 1); glVertex2f(- halfsize.x, + halfsize.y);
    glEnd;
  glPopMatrix;
end;

{ T2DCircle }

procedure T2DCircle.Draw;
var
  i: integer;
begin
  glPushMatrix;
    glTranslatef(position.x, position.y, 0);
    glBegin(GL_POINTS);
      for i := 0 to 360 do
      begin
        glVertex2f(cos(i) * radius, sin(i) * radius);
      end;
    glEnd;
  glPopMatrix;
end;

procedure DrawPolygon(shape: Tb2Shape; const xf: Tb2XForm);
var
  i: integer;
  vertices: Tb2PolyVertices;
begin
  with Tb2PolygonShape(shape) do
  begin
    for i := 0 to GetVertexCount - 1 do
      vertices[i] := b2Mul(xf, m_vertices[i]);

    glPushMatrix;
      glBegin(GL_LINE_LOOP);
      for i := 0 to GetVertexCount - 1 do
      begin
        glVertex2f(vertices[i].x, vertices[i].y);
      end;
      glEnd;
    glPopMatrix;
  end;
end;

procedure DrawCircle(shape: Tb2Shape; const xf: Tb2XForm);
var
  center: Tb2Vec2;
  i: integer;
begin
  with Tb2CircleShape(shape) do
  begin
    center := b2Mul(xf, m_localPosition);

    glPushMatrix;
      glTranslatef(center.x, center.y, 0);
      glPointSize(1);
      glBegin(GL_POINTS);
      for i := 0 to 360 do
      begin
        glVertex2f(cos(i) * GetRadius, sin(i) * GetRadius);
      end;
      glEnd;
    glPopMatrix;
  end;
end;

procedure DrawLine(x, y, l, a: double);
begin
  glPushMatrix;
    glTranslatef(x, y, 0);
    glRotatef(a, 0, 0, 1);
    glBegin(GL_LINES);
      glVertex2f(0, 0);
      glVertex2f(l, 0);
    glEnd;
  glPopMatrix;
end;

{ T2DTriangle }

procedure T2DTriangle.Draw;
begin
  glBegin(GL_TRIANGLE_FAN);
    glVertex2f(verticies[0].X, verticies[0].Y);
    glVertex2f(verticies[1].X, verticies[1].Y);
    glVertex2f(verticies[2].X, verticies[2].Y);
  glEnd;
end;

procedure T2DTriangle.params(verticies: array of TVector2D);
var
  i: integer;
begin
  if System.Length(verticies) >= 3 then
    for i := 0 to 2 do
      Self.verticies[i] := verticies[i];
end;

{ T2DSuperBox }

procedure T2DTriangleBox.Draw;
begin
  triangles[0].Draw;
  triangles[1].Draw;
end;

procedure T2DTriangleBox.params(position, Size: Tb2Vec2);
begin
  Self.position := position;
  Self.size := size;

  triangles[0].params([vec.Create(position.x, position.y), vec.Create(position.x+size.x, position.y), vec.Create(position.x, position.y+size.y)]);
  triangles[1].params([vec.Create(position.x+size.x, position.y), vec.Create(position.x+size.x, position.y+size.y), vec.Create(position.x, position.y+size.y)]);
end;

{ T2DSuperBox }

procedure T2DSuperBox.Draw;
var
  i, j: integer;
begin
  for i := 0 to System.Length(boxes) - 1 do
    for j := 0 to System.Length(boxes) - 1 do
      boxes[i*System.Length(boxes)+j].Draw;
end;

procedure T2DSuperBox.params(position, size: Tb2Vec2; boxesc: integer);
var
  i, j: integer;
begin
  SetLength(boxes, boxesc*boxesc);
  for i := 0 to boxesc - 1 do
    for j := 0 to boxesc - 1 do      
      boxes[i*boxesc+j].params(b2vec2(position.x+size.x/boxesc*i, position.y+size.y/boxesc*j), b2vec2(size.x/boxesc, size.y/boxesc));
end;

end.
