unit eng.particles;

interface
  uses OpenGL, eXgine;

const
  max_particles = 1024;

type
  TParticle = class
    x, y: single;
    life: single;
    vx, vy: single;
    ax, ay: single;
    dead: boolean;
    constructor Create(_x, _y, _vx, _vy, _ax, _ay, _life: single); virtual;
    procedure Draw; virtual;
    procedure Update;
  end;

  TNumberParticle = class(TParticle)
    num: integer;
    procedure Draw; override;
  end;

  TParticleManager = class
    particles: array[0..max_particles-1] of TParticle;
    now_length: integer;
    constructor Create;
    procedure Draw;
    procedure Update;
    procedure Add(_x, _y, _vx, _vy, _ax, _ay, _life: single); overload;
    procedure Add(_x, _y, _vx, _vy, _ax, _ay, _life: single; num: integer); overload;
  end;

implementation
  uses pin.game;

{ TParticle }

constructor TParticle.Create(_x, _y, _vx, _vy, _ax, _ay, _life: single);
begin
  x := _x; y := _y;
  vx := _vx; vy := _vy;
  ax := _ax; ay := _ay;
  life := _life;
  dead := false;
end;

procedure TParticle.Draw;
begin
  if not dead then
  begin
    glColor3f(1 * life, 1 * life, 1 * life);
    glPointSize(2);
    glPushMatrix;
      glBegin(GL_POINTS);
        glVertex2f(x, y);
      glEnd;
    glPopMatrix;
    glPointSize(1);
  end;
end;

procedure TParticle.Update;
begin
  x := x + vx;
  y := y + vy;
  vx := vx + ax;
  vy := vy + ay;
  life := life - 0.01;
  if life < 0 then
    dead := true;
end;

{ TParticleManager }

procedure TParticleManager.Add(_x, _y, _vx, _vy, _ax, _ay, _life: single);
var
  i: integer;
begin
  for i := 0 to max_particles - 1 do
    if (particles[i] = nil) or (particles[i].dead = true) then
    begin
      particles[i] := nil;
      particles[i] := TParticle.Create(_x, _y, _vx, _vy, _ax, _ay, _life);
      if now_length < i then
        now_length := i;
      break;
    end;
end;

procedure TParticleManager.Add(_x, _y, _vx, _vy, _ax, _ay, _life: single;
  num: integer);
var
  i: integer;
begin
  for i := 0 to max_particles - 1 do
    if (particles[i] = nil) or (particles[i].dead = true) then
    begin
      particles[i] := nil;
      particles[i] := TNumberParticle.Create(_x, _y, _vx, _vy, _ax, _ay, _life);
      TNumberParticle(particles[i]).num := num;
      if now_length < i then
        now_length := i;
      break;
    end;
end;

constructor TParticleManager.Create;
begin
  now_length := 0;
end;

procedure TParticleManager.Draw;
var
  i: integer;
begin
  for i := 0 to now_length - 1 do
    if (not (particles[i] = nil)) or (not (particles[i].dead)) then    
      particles[i].Draw;
end;

procedure TParticleManager.Update;
var
  i: integer;
begin
  for i := 0 to now_length - 1 do
    if (not (particles[i] = nil)) or (not (particles[i].dead)) then
    begin
      particles[i].Update;
    end;
end;

{ TNumberPartice }

procedure TNumberParticle.Draw;
begin
  inherited;
  if not dead then
  begin
    glColor3f(1 * life, 1 * life, 1 * life);

    glPushMatrix;
      glLoadIdentity;
      ogl.TextOut(0, 400+x*8, 300-(y+now_global_y)*8, PChar(IntToStr(num)));
    glPopMatrix;
    {glPointSize(2);
    glPushMatrix;
      glBegin(GL_POINTS);
        glVertex2f(x, y);
      glEnd;
    glPopMatrix;
    glPointSize(1);}
  end;  
end;

end.
