unit eng.utils;

interface
  uses eXgine, OpenGL, Windows, eng.box2d;

function RadToDeg(rad: single): single;
function DegToRad(deg: single): single;

function BetweenTwo(val, max, min: single): boolean;

function SignAngle(ang, val: single): integer;

function Distance(vec1, vec2: Tb2vec2): single;

(* \ \ *)
const
  STAT_FONT = 0;
  STAT_FONT_H = 12;
  
procedure Stat_Begin;
procedure Stat_Value(Val: integer; Str: String); overload;
procedure Stat_Value(Val: Single; Str: String; Demicals: integer = 3); overload;
procedure Stat_Value(Val: boolean; Str: String); overload;
procedure Stat_Str(Str: String);
procedure Stat_Delim;

var
  stat_count: integer;
(* / / *)

function FileExists(const FileName: string): Boolean;

implementation

function RadToDeg(rad: single): single;
begin
  Result := 180 * rad / pi;
end;

function DegToRad(deg: single): single;
begin
  Result := pi * deg / 180;
end;

function BetweenTwo(val, max, min: single): boolean;
begin
  Result := (val > min) and (val < max);
end;

function SignAngle(ang, val: single): integer;
begin
  Result := 0;
  if ang > val then
    Result := +1;
  if ang < val then
    Result := -1;
end;

function Distance(vec1, vec2: Tb2vec2): single;
begin
  Result := sqrt(sqr(vec1.X - vec2.X) + sqr(vec1.Y - vec2.Y));
end;

(* \ \ *)
procedure Stat_Begin;
begin
  stat_count := 0;
  
  glDisable(GL_DEPTH_TEST);
  tex.Disable();
  ogl.Blend(BT_NONE);
  ogl.Set2D(0, 0, wnd.Width, wnd.Height);
  glColor3f(1, 0, 0);
end;

procedure Stat_Value(Val: integer; Str: String);
begin
  inc(stat_count);
  ogl.TextOut(STAT_FONT, 8, stat_count * STAT_FONT_H, PChar(Str + ': ' + IntToStr(Val)));
end;

procedure Stat_Value(Val: Single; Str: String; Demicals: integer = 3); overload;
var
  str_val: string;
begin
  inc(stat_count);

  System.Str(Val:0:Demicals, str_val);
  ogl.TextOut(STAT_FONT, 8, stat_count * STAT_FONT_H, PChar(Str + ': ' + str_val));
end;

procedure Stat_Value(Val: boolean; Str: String); overload;
begin
  inc(stat_count);

  if Val then
    ogl.TextOut(STAT_FONT, 8, stat_count * STAT_FONT_H, PChar(Str + ': ' + 'on'))
  else
    ogl.TextOut(STAT_FONT, 8, stat_count * STAT_FONT_H, PChar(Str + ': ' + 'off'));
end;

procedure Stat_Str(Str: String);
begin
  inc(stat_count);

  ogl.TextOut(STAT_FONT, 8, stat_count * STAT_FONT_H, PChar(Str));
end;

procedure Stat_Delim;
begin
  inc(stat_count);
end;
(* / / *)

function FileExists(const FileName: string): Boolean;
var
  Code: Integer;
begin
  Code := GetFileAttributes(PChar(FileName));
  Result := (Code <> -1) and (FILE_ATTRIBUTE_DIRECTORY and Code = 0);
end;


end.
