program game;

{$Warnings off}

uses
  Windows,
  Messages,
  OpenGL,
  eXgine,
  IniFiles,
  SysUtils,
  eng.forms in 'eng.forms.pas',
  UPhysics2D,
  UPhysics2DTypes,
  eng.box2d in 'eng.box2d.pas',
  eng.utils in 'eng.utils.pas',
  pin.ball in 'pin.ball.pas',
  pin.map in 'pin.map.pas',
  pin.flipper in 'pin.flipper.pas',
  ini_params in 'ini_params.pas',
  mytex in 'mytex.pas',
  pin.tex in 'pin.tex.pas',
  pin.game in 'pin.game.pas',
  eng.particles in 'eng.particles.pas';

//{$R ico}

{$Warnings off}

var
  mouse, mouse_old: TPoint;
  life: integer;

{ --- Box2D --- }
  (*  *)
  worldAABB: Tb2AABB;  //  
  doSleep: boolean;    //   ?
  //world: Tb2World;     // 

  (*   *)
  timeStep: single;
  iterations: integer;

  contactListener: TContactListener;
  debugDraw: TDebugDraw;

  lbtn, rbtn: boolean;

  kill_ball: boolean = false;

procedure _reset(i: integer);
begin
    if flipper[i].inc_super_time >= 0 then
    begin
      flipper[i].super_time := 60;
      flipper[i].super := true;
      flipper[i].inc_super_time := -60;
      flipper[i].super_prepared := false;
    end
    else
      flipper[i].inc_super_time := -60;
end;

procedure NewGame;
begin
  worldAABB.lowerBound.x := -100; worldAABB.lowerBound.y := -100;
  worldAABB.upperBound.x := 100; worldAABB.upperBound.y := 100;
  doSleep := false;
  world := Tb2World.Create(worldAABB, gravity, doSleep);
  //   
  timeStep := 1/60;
  iterations := 10;
  //  
  contactListener := TContactListener.Create;
  world.m_contactListener := contactListener;

  debugDraw := TDebugDraw.Create;
  debugDraw.m_drawFlags := [e_shapeBit];
  world.m_debugDraw := debugDraw;

  map.Init(world);
  flipper[0].Init(world, b2vec2(0, -23));
  flipper[1].Init(world, b2vec2(0, 40));
  //now_global_y := -60;
  //floor := 2;
  ball.Init(world, b2vec2(5, -23));

  lbtn := false;
  rbtn := false;

  pin.tex.Load;

  cls := [cl_1, cl_2];

  life := 3;

end;

procedure RestartGame;
begin
  map.CloseFloor2;
  ball.Reset;
  ball.Init(world, b2vec2(5, -23));
  floor := 1;
  global_y := 0;

  world.Destroy;
  NewGame;
  life := 3;
  highscores := 0;
end;

procedure RestartBall;
begin
  map.CloseFloor2;
  ball.Reset;
  ball.Init(world, b2vec2(5, -23));
  floor := 1;
  global_y := 0;
  dec(life);
end;

procedure scores;
var
  iniFile: TIniFile;
begin
  if highscores > max_scores then
  begin
    IniFile := TIniFile.Create(ExtractFilePath(paramstr(0)) + 'data/system.ini');
    IniFile.WriteFloat('scores', 'max', highscores);
    IniFile.Free;
    max_scores := highscores;
//    max_scores := LoadParam('scores', 'max', 0.0);
    
  end;
end;

procedure OnMessage(Msg: Cardinal; wP, lP: integer);
var
  lt: TSYSTEMTIME;
begin
  case Msg of
  WM_MOUSEMOVE:
    begin
      mouse_old := mouse;
      mouse.x := LOWORD(lP);
      mouse.y := HiWord(lP);
    end;
  WM_LBUTTONDOWN:
  begin
    lbtn := true;
  end;
  WM_LBUTTONUP:
  begin
    lbtn := false;
    _reset(0);
    _reset(1);
  end;
  WM_RBUTTONDOWN:
  begin
    rbtn := true;
  end;
  WM_RBUTTONUP:
  begin
    rbtn := false;
    _reset(0);
    _reset(1);
  end;
  WM_KEYUP:
  begin
    if wP = VK_SNAPSHOT then
    begin
      GetLocalTime(lt);
      ogl.ScreenShot(PChar('..\screenshots\prototype_' +
                                  IntToStr(lt.wYear) + '.' +
                                  IntToStr(lt.wMonth) + '.' +
                                  IntToStr(lt.wDay) + '-' +
                                  IntToStr(lt.wHour) + '.' +
                                  IntToStr(lt.wMinute) + '.tga'));
    end;
    if wP = Ord('R') then
    begin
      RestartGame;
    end;
    if wP = VK_LEFT then
    begin
      lbtn := false;
      _reset(0);
      _reset(1)
    end;
    if wP = VK_RIGHT then
    begin
      rbtn := false;
      _reset(0);
      _reset(1)
    end;
  end;
  WM_KEYDOWN:
  begin
    if wP = VK_LEFT then
      lbtn := true;
    if wP = VK_RIGHT then
      rbtn := true;
  end;
  end;
end;

procedure Init;
begin
//  
  inp.MCapture(false);
  //ShowCursor(false);

  LoadPhysicsFromIni('data/physics.ini');
  LoadSystemFromIni('data/system.ini');

  //  
  NewGame;
  pman := TParticleManager.Create;

  Randomize;

//   
  wnd.Mode(false, 800, 600, 32, 75);

  glClearColor(0.0, 0.0, 0.0, 0.0);

  glFlush;
end;

procedure Free;
begin
  pman.Free;
end;

procedure Update;
var
  fl_upd, i: integer;
begin
  if inp.Down(27) then eX.Quit;
  if cam_move then
  begin
    if now_global_y < global_y then
      now_global_y := now_global_y + 1;
    if now_global_y > global_y then
      now_global_y := now_global_y - 1;
    if Round(now_global_y) = Round(global_y) then
    begin
      cam_move := false;
    end;
    exit;
  end;
  contactListener.m_pointCount := 0;
  contactListener.coll_flip := false;
  world.Step(timeStep, iterations);

  if (ball.body.GetPosition.y > 35) and (global_y <> -60) and (floor = 1) then
  begin
    cam_move := true;
    global_y := -60;
    //now_global_y := Trunc(now_global_y);
    floor := 2;
  end;
  if (ball.body.GetPosition.y < 30) and (global_y <> 0) and (floor = 2) then
  begin
    cam_move := true;
    global_y := 0;
    //now_global_y := Trunc(now_global_y);
    floor := 1;
    map.CloseFloor2;
  end;

  {if (not cam_move) and (floor = 1) then
  begin
    if (can_move_up = 2) and (ball.body.GetPosition.y < 30) then
    begin
      if (ball.body.GetPosition.y > 17) then
        global_y := -10
      else
        global_y := 0;
    end;
    if floor = 1 then
    begin
      if now_global_y < global_y then
        now_global_y := now_global_y + 0.2;
      if now_global_y > global_y then
        now_global_y := now_global_y - 0.2;
    end;
  end;}

  ball.Contact(contactListener);
  ball.Update;

  fl_upd := 1;
  if cl_2 in cls then
    fl_upd := 2;

  for i := 0 to fl_upd - 1 do
  begin
    if (i = 0) and not (cl_1 in cls) then
      continue;
    if (i = 1) and not (cl_2 in cls) then
      continue;

    if (lbtn) and (not rbtn) then
      if flipper[i].joint.GetJointAngle > flipper[i].joint.m_lowerAngle then
        flipper[i].joint.MotorSpeed := (flipper[i].joint.m_lowerAngle - flipper[i].joint.GetJointAngle) * motor_speed_koef;

    if (rbtn) and (not lbtn)  then
      if flipper[i].joint.GetJointAngle < flipper[i].joint.m_upperAngle then
        flipper[i].joint.MotorSpeed := (flipper[i].joint.m_upperAngle - flipper[i].joint.GetJointAngle) * motor_speed_koef;

    if (not rbtn) and (not lbtn) then
    begin
      flipper[i].joint.MotorSpeed := -flipper[i].joint.GetJointAngle * motor_speed_koef;
    end;
  end;

  {for i := 0 to 1 do
  begin

  if (Round(abs(flipper[i].joint.GetJointAngle)) >= Round(flipper[i].joint.m_upperAngle)) and (not flipper[i].super) then
  begin
    inc(flipper[i].inc_super_time);
    flipper[i].super_time := 0;
  end;
  if flipper[i].inc_super_time >= 0 then
  begin
    flipper[i].super_prepared := true;
    flipper[i].inc_super_time := 0;
  end;
  if flipper[i].super_time <= 0 then
    flipper[i].super := false
  else
    dec(flipper[i].super_time);

  if contactListener.coll_flip and flipper[i].super then
  begin
   // ball.body.SetLinearVelocity(b2vec2(ball.body.GetLinearVelocity.x * 2, ball.body.GetLinearVelocity.y * 2));
  end;
  end;}

  map.Update;

  pman.Update;

  if (ball.body.GetPosition.y < -31) or (ball.body.GetPosition.x < -25) or (ball.body.GetPosition.x > 25) then
    RestartBall;

  if life <= 0 then
  begin
    scores;
    RestartGame;
    RestartGame;
  end;

 { if ball.body.GetLinearVelocity.x > ball.max_velocity.x then
    ball.max_velocity.x := ball.body.GetLinearVelocity.x;
  if ball.body.GetLinearVelocity.y > ball.max_velocity.y then
    ball.max_velocity.y := ball.body.GetLinearVelocity.y;  }

  //    Escape (  27)
end;

procedure Render;
begin
  ogl.Clear(True, True);
  glColor4f(1, 1, 1, 1);
// 
  ogl.Set2D(0, 0, wnd.Width, wnd.Height);
  glLoadIdentity;

  glTranslatef(400, 300, 0);
  now_scale := -1;
  glScalef(9+now_scale, -(9+now_scale), 1);

  tex.Disable();
  //glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);

  //if cl_2 in cls then
  glTranslatef(0, now_global_y-3, 0);

  flipper[0].Draw;
  flipper[1].Draw;

  //if debug_mode.degud_draw then
    //world.DrawDebugData;

  map.Draw;
  ball.Draw;

  if draw_.particles then
    pman.Draw;

  {glColor3f(1, 1, 1);
  tex.Enable(tex_speeder);
  glBegin(GL_QUADS);
    glTexCoord2f(0, 0); glVertex2f(-5, -5);
    glTexCoord2f(1, 0); glVertex2f(+5, -5);
    glTexCoord2f(1, 1); glVertex2f(+5, +5);
    glTexCoord2f(0, 1); glVertex2f(-5, +5);
  glEnd; }

  //glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);

//  
  Stat_Begin;
  //Stat_Value(ogl.FPS, 'FPS');
  Stat_Str('Press ''R'' to restart game');
  Stat_Delim;

  {Stat_Value(flipper[0].super, 'super');
  Stat_Value(flipper[0].super_time, 'super_time');
  Stat_Value(flipper[0].inc_super_time, 'inc_super_time');
  Stat_Value(flipper[0].super_prepared, 'super_prepared');}

  if floor = 1 then
    Stat_Value(ball.health, 'health', 2);
  Stat_Value(life, 'life');

  Stat_Delim;
  {Stat_Value((mouse.X-400)/8, 'mx');
  Stat_Value((mouse.Y-300)/-8, 'my');   }

  Stat_Value(highscores, 'scores');
  Stat_Value(max_scores, 'best scores', 0);
end;

begin
  //ogl.AntiAliasing(4);
  wnd.Create(PChar('pinball'), False);
  eX.SetProc(PROC_UPDATE, @Update);
  eX.SetProc(PROC_RENDER, @Render);
  eX.SetProc(PROC_INIT, @Init);
  eX.SetProc(PROC_FREE, @Free);
  eX.SetProc(PROC_MESSAGE, @OnMessage);
//    
  eX.MainLoop(60);
end.


