unit ini_params;

interface
  uses IniFiles, eng.utils, Sysutils, eng.box2d;

var
  IniFile: TIniFile;

  // ball
  IniBall: record
    linearDamping, angularDamping: double;
    sphere_density, sphere_friction, shpere_restitution: double;

    mass, inertia: double;
  end;

  // flipper
  motor_speed_koef: double;
  flipper_restitution: double;

  // world
  gravity: Tb2Vec2;

  //map
  IniMap: record
    walls_restitution, walls_friction: double;
    cylinders_restitution: double;
    huini_restitution: double;
    holes_kickforce: double;
  end;

  (* System *)
  // debug_mode
  debug_mode: record
    degud_draw, debug_log: boolean;
  end;

  draw_: record
    particles: boolean;

  end;
  max_scores: double;

procedure LoadPhysicsFromIni(FileName: string);
procedure LoadSystemFromIni(FileName: string);

implementation

function LoadParam(const Section, Name: string; Default: Double): Double; overload;
begin
  if IniFile.ValueExists(Section, Name) then
    Result := IniFile.ReadFloat(Section, Name, Default)
  else
  begin
    IniFile.WriteFloat(Section, Name, Default);
    Result := Default;
  end;
end;

function LoadParam(const Section, Name: string; Default: boolean): boolean; overload;
begin
  if IniFile.ValueExists(Section, Name) then
    Result := IniFile.ReadBool(Section, Name, Default)
  else
  begin
    IniFile.WriteBool(Section, Name, Default);
    Result := Default;
  end;
end;

procedure LoadPhysicsFromIni(FileName: string);
begin
  IniFile := TIniFile.Create(ExtractFilePath(paramstr(0)) + FileName);

  gravity := b2vec2(LoadParam('world', 'gravity_x', 0.0), LoadParam('world', 'gravity_y', -18.8));

  IniBall.mass               := LoadParam('ball', 'mass', 4);
  IniBall.inertia            := LoadParam('ball', 'inertia', 1.5);
  IniBall.linearDamping      := LoadParam('ball', 'linear_damping', 0.2);
  IniBall.angularDamping     := LoadParam('ball', 'angular_damping', 0.0);

  IniBall.sphere_density     := LoadParam('ball', 'shpere_density', 1.0);
  IniBall.sphere_friction    := LoadParam('ball', 'shpere_friction', 0.3);
  IniBall.shpere_restitution := LoadParam('ball', 'shpere_restitution', 0.5);

  motor_speed_koef    := LoadParam('flipper', 'motor_speed_koef', 12.0);
  flipper_restitution := LoadParam('flipper', 'restitution', 2.0);

  IniMap.walls_friction    := LoadParam('map', 'walls_friction', 0.3);
  IniMap.walls_restitution := LoadParam('map', 'walls_restitution', 0.0);

  IniMap.cylinders_restitution := LoadParam('map', 'cylinders_restitution', 300.0);

  IniMap.huini_restitution := LoadParam('map', 'huini_restitution', 300.0);

  IniMap.holes_kickforce := LoadParam('map', 'holes_kickforce', 300.0);
  
  IniFile.Free;
end;

procedure LoadSystemFromIni(FileName: string);
begin
  IniFile := TIniFile.Create(ExtractFilePath(paramstr(0)) + FileName);

  debug_mode.degud_draw := LoadParam('debug_mode', 'debug_draw', true);
  debug_mode.debug_log  := LoadParam('debug_mode', 'debug_log', false);

  draw_.particles := LoadParam('draw', 'particles', true);

  max_scores := LoadParam('scores', 'max', 0.0);

  IniFile.Free;
end;

end.
