unit pin.ball;

interface
  uses UPhysics2D, eng.box2d, eng.forms, OpenGL, ini_params, UPhysics2DTypes,
      eXgine, eng.utils;

const
 max_triggers = 4;

type
  TBall = record
    body: Tb2Body;
    sh_circle: Tb2CircleShape;
    sleepTime: integer;

    health: single;
    v: Tb2vec2;
    max_velocity: Tb2vec2;
    
    imp: Tb2vec2;
    last_hole: integer;
    last_triggers: array[0..max_triggers-1] of integer;

    bsize: integer;

    procedure Init(world: Tb2World; pos: Tb2vec2);
    procedure Draw;
    procedure Reset;

    procedure Update;

    procedure Contact(contactListener: TContactListener);
  end;

var
  ball: TBall;

implementation
  uses pin.map, pin.game;

{ TBall }

procedure TBall.Contact(contactListener: TContactListener);
procedure CollideWithHoles(shape1, shape2: Tb2Shape; velocity: Tb2vec2);
var
  obj: TMapObjectHole;
begin
  if (shape2.m_body = ball.body) and (shape2 = sh_circle) and (shape1.IsSensor)
      and (sleepTime = 0) and (TMapObjectData(shape1.m_body.UserData^).type_ = mot_Hole) then
    if (Distance(shape2.m_body.GetPosition, shape1.m_body.GetPosition) <= abs(Tb2CircleShape(shape1).m_radius - Tb2CircleShape(shape2).m_radius)) then
    begin
      ball.body.PutToSleep;
      obj := TMapObjectHole(map.objects[TMapObjectData(shape1.m_body.UserData^).id]);
      imp := b2vec2(cos(DegToRad(obj.direction)) * obj.kickforce, sin(DegToRad(obj.direction)) * obj.kickforce); //velocity;
      sleepTime := 1;
      last_hole := TMapObjectData(shape1.m_body.UserData^).id;    
    end;
end;
procedure CollideWithTriggers(shape1, shape2: Tb2Shape);
var
  I, j: integer;
  _ok: boolean;
  vx, vy, ax, ay, zn: integer;
begin
  if (shape2.m_body = ball.body) and (shape2 = sh_circle) and (shape1.IsSensor)
    and (TMapObjectData(shape1.m_body.UserData^).type_ = mot_Trigger)
    and (Distance(shape2.m_body.GetPosition, shape1.m_body.GetPosition) < abs(Tb2CircleShape(shape1).m_radius + Tb2CircleShape(shape2).m_radius)) then
  begin
    _ok := true;
    j := -1;
    for i := 0 to System.Length(last_triggers) - 1 do
      if (last_triggers[i] = TMapObjectData(shape1.m_body.UserData^).id) then
        _ok := false
      else
        if last_triggers[i] = -1 then        
          j := i;
    if _ok then    
    begin
      if j = -1 then
        j := max_triggers-1;
      if not TMapObjectTrigger(map.objects[TMapObjectData(shape1.m_body.UserData^).id]).empty then
        TMapObjectTrigger(map.objects[TMapObjectData(shape1.m_body.UserData^).id]).active := not TMapObjectTrigger(map.objects[TMapObjectData(shape1.m_body.UserData^).id]).active;
      last_triggers[j] := TMapObjectData(shape1.m_body.UserData^).id;

    if TMapObjectTrigger(map.objects[TMapObjectData(shape1.m_body.UserData^).id]).empty then
    begin
      for i := 0 to ((TMapObjectTrigger(map.objects[TMapObjectData(shape1.m_body.UserData^).id]).cost) div 250)+3 do
      begin
        zn := random(2);
        if zn = 0 then
          zn := -1;
        vx := zn * random(100);
        zn := random(2);
        if zn = 0 then
          zn := -1;
        vy := zn * random(100);
        zn := random(2);
        if zn = 0 then
          zn := -1;
        ax := zn * random(100);
        zn := random(2);
        if zn = 0 then
          zn := -1;
        ay := zn * random(100);
        pman.Add(shape2.m_body.GetPosition.x, shape2.m_body.GetPosition.y,
                  vx/2000 , vy/2000,
                  ax/20000, -ay/20000,
                  1, 250);
      end;
      inc(highscores, TMapObjectTrigger(map.objects[TMapObjectData(shape1.m_body.UserData^).id]).cost);
    end;
    end;    
  end;
end;
var
  i: integer;
begin
  for i := 0 to contactListener.m_pointCount - 1 do
    with contactListener.m_points[i].b2point do
    begin
      CollideWithHoles(shape1, shape2, velocity);
      CollideWithHoles(shape2, shape1, velocity);

      CollideWithTriggers(shape1, shape2);
      CollideWithTriggers(shape2, shape1);

//      CollideWithCylinders(contactListener.m_points[i]);
    end;
end;

procedure TBall.Draw;
var
  center: Tb2Vec2;
  i: integer;
  r: single;
begin
  glColor3f(1, 1, 1);
  DrawCircle(sh_circle, body.m_xf);
end;

procedure TBall.Init;
var
  bodyDef: Tb2BodyDef;
  circleDef: Tb2CircleDef;
  i: integer;
  data: TMapObjectData;
begin
  bodyDef := Tb2BodyDef.Create;
  bodyDef.position := pos;
  bodyDef.angle := 0;
  bodyDef.isBullet := true;
  bodyDef.linearDamping := IniBall.linearDamping;
  bodyDef.angularDamping := IniBall.angularDamping;

  if IniBall.mass <> -1 then
  begin
    bodyDef.massData.mass := IniBall.mass;
    bodyDef.massData.I := InIBall.inertia;
    SetZero(bodyDef.massData.center);
  end;

  body := world.CreateBody(bodyDef);

  circleDef := Tb2CircleDef.Create;
  circleDef.radius := 1.5;
  circleDef.density := IniBall.sphere_density;
  circleDef.friction := IniBall.sphere_friction;
  circleDef.restitution := IniBall.shpere_restitution;
  circleDef.localPosition.x := 0; circleDef.localPosition.y := 0;
  sh_circle := Tb2CircleShape(body.CreateShape(circleDef));

  if IniBall.mass = -1 then
    body.SetMassFromShapes;

  body.PutToSleep;

  health := 50;

  max_velocity := b2vec2(100, 100);
  bsize := 2;

  data.type_ := mot_Null;
  body.UserData := @data;

  sleepTime := 0;
  last_hole := -1;
  //SetLength(last_triggers, 1);
  for i := 0 to max_triggers - 1 do  
    last_triggers[i] := -1;
end;

procedure TBall.Reset;
begin
  body.Free;
  sh_circle := nil;
end;

function FindGroup(id: integer): integer;
var
  i, j: integer;
begin
  Result := -1;
  for i := 0 to map.groups_count - 1 do
    for j := 0 to System.Length(map.groups[i].elements) - 1 do
      if map.groups[i].elements[j].id = id then
      begin
        Result := i;
        break;
      end;
end;

procedure TBall.Update;
var
  i, group, mulx: integer;
    vx, vy, ax, ay: integer;
begin
  if sleepTime > 0 then
  begin
    inc(sleepTime);
    ball.body.PutToSleep;
  end;
  if sleepTime > 100 then
  begin
    ball.health := ball.health - map.objects[last_hole].data.dam;
    body.WakeUp;
    body.ApplyImpulse(imp, body.GetPosition);
    sleepTime := -1;

    // Highscores
      group := FindGroup(last_hole);
      if group > -1 then
      begin
        mulx := map.groups[group].CountLikeHoles;
        inc(highscores, map.objects[last_hole].cost * mulx);
      end;

    for i := 0 to 19 do
    begin
      vx := random(100);
      vy := random(100);
      ax := random(100);
      ay := random(100);
      pman.Add(body.GetPosition.x, body.GetPosition.y,
               vx/1000 , vy/1000,
               ax/10000, -ay/10000, 1.5);
    end; 
  end;
  if sleepTime = -1 then
    if last_hole <> -1 then
      if Distance(body.GetPosition, map.objects[last_hole].body.GetPosition) >
       abs(Tb2CircleShape(sh_circle).m_radius + Tb2CircleShape(map.objects[last_hole].shape).m_radius) then
        sleepTime := 0;

  for i := 0 to max_triggers - 1 do
    if last_triggers[i] <> -1 then
      if Distance(body.GetPosition, map.objects[last_triggers[i]].body.GetPosition) >
          abs(Tb2CircleShape(sh_circle).m_radius + Tb2CircleShape(map.objects[last_triggers[i]].shape).m_radius) then
        last_triggers[i] := -1;
  
  if health <= 0 then
  begin
    case bsize of
      1:
      begin
        //Tb2CircleShape(body.m_shapeList).m_radius := 1.0;
      end;
      2:
      begin
        Tb2CircleShape(body.m_shapeList).m_radius := 1.2;
      end;
    end;
    dec(bsize);
    if bsize <= 1 then
      bsize := 1;
    health := 50;
    if floor = 1 then   
      map.OpenFloor2;

    {for i := 0 to 99 do
    begin
      vx := random(100);
      vy := random(100);
      ax := random(100);
      ay := random(100);
      pman.Add(body.GetPosition.x, body.GetPosition.y,
               vx/1000 , vy/1000,
               ax/10000, -ay/10000, 1.5);
    end; }
  end;

end;

end.
