unit pin.flipper;

interface
  uses UPhysics2D, OpenGL, eng.forms, eng.box2d, eng.utils, ini_params,
    pin.map, pin.tex, eXgine;

type
  TFlipper = record
    pivotBody, flippBody: Tb2Body;
    sh_pivot: Tb2CircleShape;
    sh_flipp: Tb2PolygonShape;
    joint: Tb2RevoluteJoint;
    data: TMapObjectData;

    super: boolean;
    super_time: integer;
    inc_super_time: integer;
    super_prepared: boolean;

    procedure Init(world: Tb2World; pos: Tb2vec2);
    procedure Draw;
  end;

var
  flipper: array[0..1] of TFlipper;

implementation

{ TFlipper }

procedure TFlipper.Draw;
begin
  glColor3f(1, 0, 0);
  DrawCircle(sh_pivot, pivotBody.m_xf);
  DrawPolygon(sh_flipp, flippBody.m_xf);
end;

procedure TFlipper.Init;
var
  BodyDef: Tb2BodyDef;
  flippShapeDef: Tb2PolygonDef;
  pivotShapeDef: Tb2CircleDef;
  jointDef: Tb2RevoluteJointDef;
begin
  BodyDef := Tb2BodyDef.Create;
  BodyDef.position := pos;
  pivotBody := world.CreateBody(BodyDef);
  pivotShapeDef := Tb2CircleDef.Create;
  pivotShapeDef.radius := 0.1;
  pivotShapeDef.density := 0;
  sh_pivot := Tb2CircleShape(pivotBody.CreateShape(pivotShapeDef));
  pivotBody.SetMassFromShapes;
  pivotBody.SetBullet(true);

  BodyDef := Tb2BodyDef.Create;
  BodyDef.position := pos;
  flippBody := world.CreateBody(BodyDef);
  flippShapeDef := Tb2PolygonDef.Create;
  flippShapeDef.SetAsBox(0.5, 4.5, b2vec2(0, -2.25), 0);
  flippShapeDef.density := 1;
  flippShapeDef.restitution := flipper_restitution;
  //flippShapeDef.isSensor := true;
  sh_flipp := Tb2PolygonShape(flippBody.CreateShape(flippShapeDef));
  flippBody.SetMassFromShapes;
  flippBody.SetBullet(true);

  data.type_ := mot_Flip;
  flippBody.UserData := @data;

  jointDef := Tb2RevoluteJointDef.Create;
  jointDef.Initialize(pivotBody, flippBody, pivotBody.GetWorldCenter);
  jointDef.enableLimit := true;
  jointDef.motorSpeed := 0;
  jointDef.maxMotorTorque := 100000;
  jointDef.enableMotor := true;
  jointDef.lowerAngle := -DegToRad(125);
  jointDef.upperAngle := DegToRad(125);

  joint := Tb2RevoluteJoint(world.CreateJoint(jointDef));

  super := false;
  super_time := 0;
  inc_super_time := -60;
end;

end.
