unit pin.map;

interface
  uses UPhysics2D, OpenGL, eng.forms, eng.box2d, eng.utils, ini_params, eXgine,
    pin.tex, pin.game;

type
  TMapObjectType = (mot_Null, mot_Cylinder, mot_Hole, mot_Trigger, mot_Huinya,
                    mot_Flip, mot_Speeder, mot_Wall, mot_Kick);

  TMapObjectData = record
    type_: TMapObjectType;
    id: integer;
    dam: single;
  end;

  TMapObject = class
    data: TMapObjectData;
    body: Tb2Body;
    shape: Tb2Shape;
    cost: integer;
    procedure Draw; virtual; abstract;
  end;

  TMapObjectCylinder = class(TMapObject)
    color: integer; // 0 - yellow; 1 - green; 2 - blue; 3 - red; 

    constructor Create(world: Tb2World; x, y, radius: double; id: integer);
    procedure Draw; override;
  end;

  TMapObjectHole = class(TMapObject)
    direction: double;
    kickforce: double;
    shapeBox: array[0..2 ] of Tb2Shape;

    constructor Create(world: Tb2World; x, y, radius: double; direction: double; id: integer);
    procedure Draw; override;
  end;

  TMapObjectTrigger = class(TMapObject)
    active: boolean;
    empty: boolean;


    constructor Create(world: Tb2World; x, y, radius: double; empty_: boolean; id: integer);
    procedure Draw; override;
  end;

  TMapObjectHuinya = class(TMapObject)
    sh_proujin: Tb2PolygonShape;

    constructor Create(world: Tb2World; x, y: double; verticies: array of Tb2vec2; proujin_s, proujin_e: integer; angle: double; id: integer);
    procedure Draw; override;
  end;

  TMapObjectSpeeder = class(TMapObject)
    direction: double;
    speed_force: double;
    active: boolean;
    hs: tb2vec2;

    constructor Create(world: Tb2World; pos, halfsize: Tb2vec2; angle: double; id: integer);
    procedure Draw; override;
  end;

  TMapObjectKicker = class(TMapObject)
    constructor Create(world: Tb2World; x, y: double; hs: Tb2vec2; ang: double; id: integer);
    procedure Draw; override;
  end;

  TScoreElement = record
    id: integer;
    t: TMapObjectType;
  end;

  TScoresGroup = record
    elements: array of TScoreElement;

    procedure AddElement(t: TMapObjectType; id: integer);

    function CountLikeHuini: integer;
    function CountLikeCylinders(id: integer): integer;
    function CountLikeHoles: integer;
  end;

  TMap = record
    bodies: array[0..99] of Tb2Body;
    shapes: array[0..99] of Tb2PolygonShape;

    objects: array of TMapObject;
    groups: array[0..99] of TScoresGroup;
    groups_count: integer;

    procedure Init(world: Tb2World);
    procedure Draw;
    procedure Update;
    procedure OpenFloor2;
    procedure CloseFloor2;
  end;

var
  map: TMap;

implementation
  uses pin.ball;

{ TMap }


procedure TMap.CloseFloor2;
begin
  TMapObjectSpeeder(objects[7]).active := false;
  TMapObjectSpeeder(objects[8]).active := false;
  TMapObjectSpeeder(objects[9]).active := false;
  can_move_up := 1;
  map.shapes[1].IsSensor := false;
end;

procedure TMap.Draw;
var
  i: integer;
begin
  glColor3f(0.5, 0.9, 0.5);
  for i := 0 to length(bodies)-1 do
    if bodies[i] <> nil then
      if not shapes[i].IsSensor then      
        DrawPolygon(shapes[i], bodies[i].m_xf);
  for i := 0 to System.Length(objects) - 1 do
    if objects[i] <> nil then
      //if (objects[i].data.type_ = mot_Trigger) or (objects[i].data.type_ = mot_Cylinder) then
        objects[i].Draw;
  if debug_mode.degud_draw then
  begin
    for i := 0 to System.Length(objects) - 1 do
      if objects[i] <> nil then
        case objects[i].data.type_ of
          mot_Hole:
          begin
            glColor3f(0.5, 0.9, 0.5);
            DrawLine(objects[i].body.GetPosition.x, objects[i].body.GetPosition.y, Tb2CircleShape(objects[i].shape).GetRadius, TMapObjectHole(objects[i]).direction);
          end;
          mot_Speeder:
          begin
            if TMapObjectSpeeder(objects[i]).active then
            begin
              glColor3f(0.9, 0.5, 0.5);
            DrawLine(objects[i].body.GetPosition.x, objects[i].body.GetPosition.y, TMapObjectSpeeder(objects[i]).hs.x, TMapObjectSpeeder(objects[i]).direction);

            end;
            {glPushMatrix;
              glColor3f(1, 1, 1);
              tex.Enable(tex_speeder);
              glTranslatef(objects[i].body.GetPosition.x, objects[i].body.GetPosition.y, 0);
              glRotatef(TMapObjectSpeeder(objects[i]).direction-90, 0, 0, 1);
              glBegin(GL_QUADS);
              glTexCoord2f(0, 0); glVertex2f(-TMapObjectSpeeder(objects[i]).hs.x*1.5, -TMapObjectSpeeder(objects[i]).hs.y*1.5);
              glTexCoord2f(1, 0); glVertex2f(+TMapObjectSpeeder(objects[i]).hs.x*1.5, -TMapObjectSpeeder(objects[i]).hs.y*1.5);
              glTexCoord2f(1, 1); glVertex2f(+TMapObjectSpeeder(objects[i]).hs.x*1.5, +TMapObjectSpeeder(objects[i]).hs.y*1.5);
              glTexCoord2f(0, 1); glVertex2f(-TMapObjectSpeeder(objects[i]).hs.x*1.5, +TMapObjectSpeeder(objects[i]).hs.y*1.5);
              glEnd;
              tex.Disable();
            glPopMatrix; }

          end;
        end;
  end;
end;

procedure TMap.Init;
const
  floor_1 = 0;
  floor_2 = 12;
var
  i: integer;
  map: array[0..40] of T2DBox;
  groundBodyDef: Tb2BodyDef;
  groundShapeDef: Tb2PolygonDef;

  mcount: integer;
  data: TMapObjectData;
begin
  // First Floor
  {map[0].position := b2vec2(0, -31);
  map[0].halfsize := b2vec2(22, 0.5);
  map[0].angle := 0;   }
  map[1].position := b2vec2(0, 31);
  map[1].halfsize := b2vec2(22, 0.5);
  map[1].angle := 0;
  map[2].position := b2vec2(22, 7);
  map[2].halfsize := b2vec2(24, 0.5);
  map[2].angle := 90;
  map[3].position := b2vec2(-22, 7);
  map[3].halfsize := b2vec2(24, 0.5);
  map[3].angle := 90;

  map[4].position := b2vec2(-15, -20);
  map[4].halfsize := b2vec2(7.5, 0.5);
  map[4].angle := -30;
  map[5].position := b2vec2(15, -20);
  map[5].halfsize := b2vec2(7.5, 0.5);
  map[5].angle := 30;

  map[6].position := b2vec2(-18, 15);
  map[6].halfsize := b2vec2(3, 0.5);
  map[6].angle := 38;
  map[7].position := b2vec2(18, 15);
  map[7].halfsize := b2vec2(0, 0.0);
  map[7].angle := -50;

  map[8].position := b2vec2(-12, 27);
  map[8].halfsize := b2vec2(5.5, 0.5);
  map[8].angle := 38;
  map[9].position := b2vec2(13, 24.5);
  map[9].halfsize := b2vec2(10, 0.5);
  map[9].angle := -38;

  map[10].position := b2vec2(19, 8);
  map[10].halfsize := b2vec2(3, 0.5);
  map[10].angle := -38;
  map[11].position := b2vec2(19, 11.5);
  map[11].halfsize := b2vec2(3, 0.5);
  map[11].angle := 48;

  // Second Floor
  map[floor_2+1].position := b2vec2(-20, 56);
  map[floor_2+1].halfsize := b2vec2(0.5, 25);
  map[floor_2+1].angle := 0;
  map[floor_2+2].position := b2vec2(20, 56);
  map[floor_2+2].halfsize := b2vec2(0.5, 25);
  map[floor_2+2].angle := 0;

  map[floor_2+3].position := b2vec2(0, 81);
  map[floor_2+3].halfsize := b2vec2(20, 0.5);
  map[floor_2+3].angle := 0;

  map[floor_2+4].position := b2vec2(14, 40);
  map[floor_2+4].halfsize := b2vec2(8, 0.5);
  map[floor_2+4].angle := 30;

  map[floor_2+5].position := b2vec2(-12, 40);
  map[floor_2+5].halfsize := b2vec2(6, 0.5);
  map[floor_2+5].angle := -45;
  map[floor_2+6].position := b2vec2(-16, 50);
  map[floor_2+6].halfsize := b2vec2(0.5, 6);
  map[floor_2+6].angle := 0;

  {map[floor_2+7].position := b2vec2(-10, 70);
  map[floor_2+7].halfsize := b2vec2(3, 0.5);
  map[floor_2+7].angle := 30;}
  {map[floor_2+8].position := b2vec2(10, 70);
  map[floor_2+8].halfsize := b2vec2(3, 0.5);
  map[floor_2+8].angle := -30;}

  // Hole
  {map[floor_2+9].position := b2vec2(10.5, 64);
  map[floor_2+9].halfsize := b2vec2(0.3, 4.4);
  map[floor_2+9].angle := -32;
  map[floor_2+10].position := b2vec2(14, 62);
  map[floor_2+10].halfsize := b2vec2(0.3, 4.4);
  map[floor_2+10].angle := -27; }

  map[floor_2+11].position := b2vec2(6.5, 33.2);
  map[floor_2+11].halfsize := b2vec2(0.5, 3.5);
  map[floor_2+11].angle := -18;
  map[floor_2+12].position := b2vec2(-8.0, 33.2);
  map[floor_2+12].halfsize := b2vec2(0.5, 3);
  map[floor_2+12].angle := -0;

  map[floor_2+13].position := b2vec2(-16, 38);
  map[floor_2+13].halfsize := b2vec2(6, 0.5);
  map[floor_2+13].angle := -45;
  map[floor_2+14].position := b2vec2(-16.3, 30.2);
  map[floor_2+14].halfsize := b2vec2(0.5, 6);
  map[floor_2+14].angle := -45;
  map[floor_2+15].position := b2vec2(-20.5, 23.5);
  map[floor_2+15].halfsize := b2vec2(0.5, 2.5);
  map[floor_2+15].angle := 5;

  map[floor_2+16].position := b2vec2(17.5, 79.5);
  map[floor_2+16].halfsize := b2vec2(3, 0.5);
  map[floor_2+16].angle := -30;

  map[floor_2+17].position := b2vec2(17.5, 50);
  map[floor_2+17].halfsize := b2vec2(0.3, 4.4);
  map[floor_2+17].angle := -37;
  {map[floor_2+18].position := b2vec2(18, 57);
  map[floor_2+18].halfsize := b2vec2(0.3, 2.5);
  map[floor_2+18].angle := 53;}

  mcount := floor_2+18;

  for i := 0 to mcount do
  begin
    groundBodyDef := Tb2BodyDef.Create;
    groundBodyDef.position := map[i].position;
    groundBodyDef.angle := DegToRad(map[i].angle);
    bodies[i] := world.CreateBody(groundBodyDef);
    groundShapeDef := Tb2PolygonDef.Create;
    groundShapeDef.SetAsBox(map[i].halfsize.x, map[i].halfsize.y);
    groundShapeDef.friction := IniMap.walls_friction;
    groundShapeDef.restitution := IniMap.walls_restitution;
    shapes[i] := Tb2PolygonShape(bodies[i].CreateShape(groundShapeDef));
    data.type_ := mot_Wall;
    bodies[i].UserData := @data;
  end;

  //bodies[1].Destroy;

  SetLength(objects, 50);

  groups_count := 5;

  (*  0 *)
  objects[0] := TMapObjectHuinya.Create(world, -18, -17, [b2vec2(6, 0), b2vec2(0, 12), b2vec2(0, 4)], 0, 1, -63, 0);
  objects[1] := TMapObjectHuinya.Create(world, 18, -17, [b2vec2(-6, 0), b2vec2(0, 4), b2vec2(0, 12)], 0, 2, 63, 1);
  objects[29] := TMapObjectTrigger.Create(world, -12, -9, 1, false, 29);
  objects[30] := TMapObjectTrigger.Create(world, 12, -9, 1, false, 30);
  groups[0].AddElement(mot_Huinya, 0);
  groups[0].AddElement(mot_Huinya, 1);
  groups[0].AddElement(mot_Trigger, 29);
  groups[0].AddElement(mot_Trigger, 30);
  objects[0].cost := 50;
  objects[1].cost := 50;
  (* --- *)

  (*  1 *)
  objects[2] := TMapObjectCylinder.Create(world, -4, 6, 2.1, 2);
  objects[3] := TMapObjectCylinder.Create(world, +4, 6, 2.1, 3);
  objects[4] := TMapObjectCylinder.Create(world, -3.5, 14, 1.5, 4);
  objects[5] := TMapObjectCylinder.Create(world, +3.5, 14, 1.5, 5);
  objects[2].cost := 70;
  objects[3].cost := 70;
  objects[4].cost := 30;
  objects[5].cost := 30;

  groups[1].AddElement(mot_Cylinder, 2);
  groups[1].AddElement(mot_Cylinder, 3);
  groups[1].AddElement(mot_Cylinder, 4);
  groups[1].AddElement(mot_Cylinder, 5);

  objects[31] := TMapObjectTrigger.Create(world, -4, 10, 1, false, 31);
  objects[32] := TMapObjectTrigger.Create(world, 4, 10, 1, false, 32);
  objects[33] := TMapObjectTrigger.Create(world, 0, 14, 1, false, 33);
  objects[34] := TMapObjectTrigger.Create(world, 0, 6, 1, false, 34);

  groups[1].AddElement(mot_Trigger, 31);
  groups[1].AddElement(mot_Trigger, 32);
  groups[1].AddElement(mot_Trigger, 33);
  groups[1].AddElement(mot_Trigger, 34);
  (* --- *)

  (*  2 -  *)
  objects[6] := TMapObjectHole.Create(world, -20, 17, 2.0, 38, 6);
  objects[6].cost := 1000;
  groups[2].AddElement(mot_Hole, 6);
  (* --- *)

  // 
  objects[7] := TMapObjectSpeeder.Create(world, b2vec2(0, 25), b2vec2(3, 3), 90, 7);
  objects[8] := TMapObjectSpeeder.Create(world, b2vec2(-5, 25), b2vec2(3, 3), 60, 8);
  objects[9] := TMapObjectSpeeder.Create(world, b2vec2(5, 25), b2vec2(3, 3), 120, 9);

  //    
  objects[14] := TMapObjectKicker.Create(world, 19, 8, b2vec2(3, 0.9), -38, 14);
  objects[15] := TMapObjectKicker.Create(world, 19, 11.5, b2vec2(3, 0.9), 48, 15);
  objects[16] := TMapObjectKicker.Create(world, -17.5, 14.5, b2vec2(3, 0.5), 38, 16);
  objects[14].cost := 20;
  objects[15].cost := 20;
  objects[16].cost := 20;

  // Floor 2
  (*  3 *)
  objects[10] := TMapObjectHole.Create(world, 17, 45, 1.8, -150, 10);
  objects[10].cost := 5000;
  objects[35] := TMapObjectTrigger.Create(world, 13, 43, 1, false, 35);
  groups[3].AddElement(mot_Hole, 10);
  groups[3].AddElement(mot_Trigger, 35);
  (* --- *)

  //    
  objects[12] := TMapObjectKicker.Create(world, 17.3, 79, b2vec2(3.2, 0.3), -30, 12);
  objects[19] := TMapObjectKicker.Create(world, 17.5, 50, b2vec2(0.5, 5.0), -37, 19);
  objects[45] := TMapObjectKicker.Create(world, 17.4, 50.2, b2vec2(0.3, 4.7), -38, 45);
  objects[12].cost := 40;
  objects[12].cost := 40;

  objects[17] := TMapObjectHuinya.Create(world, -15.5, 41.5, [b2vec2(3, 0), b2vec2(0, 12), b2vec2(0, 2.6)], 0, 1, -75, 17);
  objects[17].cost := 100;

  (*  4 *)                                                              
  objects[18] := TMapObjectCylinder.Create(world, -18, 79, 2.5, 18);
  objects[19] := TMapObjectCylinder.Create(world, 10, 65, 1.5, 19);
  objects[20] := TMapObjectCylinder.Create(world, 5, 69, 1.5, 20);
  objects[21] := TMapObjectCylinder.Create(world, -5, 60, 2, 21);
  objects[22] := TMapObjectCylinder.Create(world, -10, 65, 1.5, 22);
  objects[23] := TMapObjectCylinder.Create(world, -5, 69, 1.5, 23);

  objects[18].cost := 500;
  objects[19].cost := 100;
  objects[20].cost := 100;
  objects[21].cost := 250;
  objects[22].cost := 150;
  objects[23].cost := 150;

  groups[4].AddElement(mot_Cylinder, 18);
  groups[4].AddElement(mot_Cylinder, 19);
  groups[4].AddElement(mot_Cylinder, 20);
  groups[4].AddElement(mot_Cylinder, 21);
  groups[4].AddElement(mot_Cylinder, 22);
  groups[4].AddElement(mot_Cylinder, 23);
  
  objects[36] := TMapObjectTrigger.Create(world, -8, 68, 1, false, 36);
  objects[37] := TMapObjectTrigger.Create(world, -5, 65, 1, false, 37);
  objects[40] := TMapObjectTrigger.Create(world, -8, 62, 1, false, 40);
  objects[38] := TMapObjectTrigger.Create(world, 6.5, 66, 1, false, 38);
  objects[39] := TMapObjectTrigger.Create(world, 8, 68, 1, false, 39);
  objects[41] := TMapObjectTrigger.Create(world, -18, 74, 1, false, 41);
  objects[42] := TMapObjectTrigger.Create(world, -13, 79, 1, false, 42);

  groups[4].AddElement(mot_Trigger, 36);
  groups[4].AddElement(mot_Trigger, 37);
  groups[4].AddElement(mot_Trigger, 38);
  groups[4].AddElement(mot_Trigger, 39);
  groups[4].AddElement(mot_Trigger, 40);
  groups[4].AddElement(mot_Trigger, 41);
  groups[4].AddElement(mot_Trigger, 42);
  (* --- *)

  // 
  objects[24] := TMapObjectTrigger.Create(world, 0, 10, 2, true, 24);//center
  TMapObjectTrigger(objects[24]).cost := 5000;
  objects[25] := TMapObjectTrigger.Create(world, 19.7, -10, 1.3, true, 25); //right+500
  TMapObjectTrigger(objects[25]).cost := 1000;
  objects[26] := TMapObjectTrigger.Create(world, 15.5, -17, 1, true, 26); //right+100
  TMapObjectTrigger(objects[26]).cost := 250;
  objects[27] := TMapObjectTrigger.Create(world, -19.7, -10, 1.3, true, 27); //l+500
  TMapObjectTrigger(objects[27]).cost := 1000;
  objects[28] := TMapObjectTrigger.Create(world, -15.5, -17, 1, true, 28); //l+100
  TMapObjectTrigger(objects[28]).cost := 250;
  objects[43] := TMapObjectTrigger.Create(world, -18, 58, 1, true, 43); //+2000
  TMapObjectTrigger(objects[43]).cost := 2000;

end;

procedure TMap.OpenFloor2;
begin    
  TMapObjectSpeeder(objects[7]).active := true;
  TMapObjectSpeeder(objects[8]).active := true;
  TMapObjectSpeeder(objects[9]).active := true;
  can_move_up := 2;
  map.shapes[1].IsSensor := true;
end;

procedure TMap.Update;
begin
end;

constructor TMapObjectCylinder.Create;
var
  bodyDef: Tb2BodyDef;
  shapeDef: Tb2CircleDef;
begin
  bodyDef := Tb2BodyDef.Create;
  bodyDef.position := b2vec2(x, y);
  body := world.CreateBody(bodyDef);

  // 
  shapeDef := Tb2CircleDef.Create;
  shapeDef.radius := radius/1.1;
  shapeDef.restitution := 0;
  shapeDef.isSensor := false;
  shape := Tb2CircleShape(body.CreateShape(shapeDef));

  // 
  shapeDef := Tb2CircleDef.Create;
  shapeDef.radius := radius;
  shapeDef.restitution := IniMap.cylinders_restitution;
  shapeDef.isSensor := true;

  shape := Tb2CircleShape(body.CreateShape(shapeDef));

  data.dam := 5;
  data.id := id;
  data.type_ := mot_Cylinder;

  color := 0;

  body.UserData := @data;
end;

procedure TMapObjectCylinder.Draw;
begin
  case color of
    0: glColor3f(1, 1, 0);
    1: glColor3f(0, 1, 0);
    2: glColor3f(0, 0, 1);
    3: glColor3f(1, 0, 0);
  end;
  DrawLine(body.GetPosition.x, body.GetPosition.y, Tb2CircleShape(shape).m_radius, 0);
//  DrawCircle(shape, body.m_xf);
  glColor3f(0.5, 0.9, 0.5);
  DrawCircle(shape, body.m_xf);
end;
    
{ TMapObjectHole }

constructor TMapObjectHole.Create;
var
  bodyDef: Tb2BodyDef;
  shapeDef: Tb2CircleDef;

  shapeBoxDef: Tb2PolygonDef;
begin
  bodyDef := Tb2BodyDef.Create;
  bodyDef.position := b2vec2(x, y);
  body := world.CreateBody(bodyDef);
  shapeDef := Tb2CircleDef.Create;
  shapeDef.radius := radius;
  shapeDef.friction := 0;
  shapeDef.restitution := 0;
  shapeDef.isSensor := true;
  shape := Tb2CircleShape(body.CreateShape(shapeDef));
  Self.direction := direction;
  kickforce := IniMap.holes_kickforce;

  data.type_ := mot_Hole;
  data.id := id;
  data.dam := 10;
  
  body.UserData := @data;

  radius := radius * 1.1;
  shapeBoxDef := Tb2PolygonDef.Create;
  shapeBoxDef.SetAsBox(radius, 0.3, b2vec2(-cos(DegToRad(direction))*radius, -sin(DegToRad(direction))*radius), DegToRad(direction+90));
  shapeBoxDef.restitution := IniMap.walls_restitution;
  shapeBox[0] := Tb2PolygonShape(body.CreateShape(shapeBoxDef));

  shapeBoxDef := Tb2PolygonDef.Create;
  shapeBoxDef.SetAsBox(radius, 0.3, b2vec2(-cos(DegToRad(direction+90))*radius, -sin(DegToRad(direction+90))*radius), DegToRad(direction));
  shapeBoxDef.restitution := IniMap.walls_restitution;
  shapeBox[1] := Tb2PolygonShape(body.CreateShape(shapeBoxDef));

  shapeBoxDef := Tb2PolygonDef.Create;
  shapeBoxDef.SetAsBox(radius, 0.3, b2vec2(-cos(DegToRad(direction-90))*radius, -sin(DegToRad(direction-90))*radius), DegToRad(direction));
  shapeBoxDef.restitution := IniMap.walls_restitution;
  shapeBox[2] := Tb2PolygonShape(body.CreateShape(shapeBoxDef));
end;

procedure TMapObjectHole.Draw;
var
  i: integer;
begin
  glColor3f(0.3, 0.3, 1);
  DrawCircle(shape, body.m_xf);

  DrawLine(body.GetPosition.x, body.GetPosition.y, Tb2CircleShape(shape).GetRadius, direction);

  glColor3f(0.5, 0.9, 0.5);
  for i := 0 to 2 do
    DrawPolygon(shapeBox[i], body.m_xf);
end;

{ TMapObjectTrigger }

constructor TMapObjectTrigger.Create(world: Tb2World; x, y, radius: double;
  empty_: boolean; id: integer);
var
  bodyDef: Tb2BodyDef;
  shapeDef: Tb2CircleDef;
begin
  bodyDef := Tb2BodyDef.Create;
  bodyDef.position := b2vec2(x, y);
  body := world.CreateBody(bodyDef);
  shapeDef := Tb2CircleDef.Create;
  shapeDef.radius := radius;
  shapeDef.friction := 0;
  shapeDef.restitution := 0;
  shapeDef.isSensor := true;
  shape := Tb2CircleShape(body.CreateShape(shapeDef));

  data.type_ := mot_Trigger;
  data.id := id;
  empty := empty_;
  body.UserData := @data;

  active := false;
end;

procedure TMapObjectTrigger.Draw;
begin
  inherited;
  if not empty then
  begin
  if active then
    glColor3f(0, 1, 1)
  else
    glColor3f(0, 0.3, 0.3);
  DrawCircle(shape, body.m_xf);
  end;
  if empty then
  begin
      glColor3f(1, 0, 1);
    DrawCircle(shape, body.m_xf);
    {glPushMatrix;
      glColor3f(1, 1, 1);
      glLoadIdentity;
      ogl.TextOut(0, 400+body.GetPosition.x*8, 300-(body.GetPosition.y+now_global_y-3)*8, PChar(IntToStr(cost)));
    glPopMatrix;}
  end;
end;

{ TMapObjectHuinya }

constructor TMapObjectHuinya.Create;
var
  bodyDef: Tb2BodyDef;
  shapeDef: Tb2PolygonDef;
  i: integer;
  hx, hy: double;
begin
  bodyDef := Tb2BodyDef.Create;
  bodyDef.position := b2vec2(x, y);
  body := world.CreateBody(bodyDef);

  shapeDef := Tb2PolygonDef.Create;
  hx := sqrt(sqr(verticies[proujin_s].x-verticies[proujin_e].x) + sqr(verticies[proujin_e].y-verticies[proujin_s].y))/2.2;
  hy := 0.25;
  angle := DegToRad(angle);
  shapeDef.SetAsBox(hx, hy, b2vec2((verticies[proujin_s].x-verticies[proujin_e].x)/2, (verticies[proujin_e].y-verticies[proujin_s].y)/2), angle);
  shapeDef.isSensor := true;
  shapeDef.restitution := IniMap.huini_restitution;
  sh_proujin := Tb2PolygonShape(body.CreateShape(shapeDef));

  shapeDef := Tb2PolygonDef.Create;
  shapeDef.restitution := IniMap.walls_restitution;
  shapeDef.friction := IniMap.walls_friction;
  shapeDef.vertexCount := 3;
  for i := 0 to 2 do
    shapeDef.vertices[i] := verticies[i];
  shape := Tb2PolygonShape(body.CreateShape(shapeDef));
  body.SetMassFromShapes;

  data.dam := 5;
  data.id := id;
  data.type_ := mot_Huinya;

  body.UserData := @data;
end;

procedure TMapObjectHuinya.Draw;
begin
  glColor3f(0.5, 0.9, 0.5);
  DrawPolygon(shape, body.m_xf);

  glColor3f(0.3, 0.3, 1);
  DrawPolygon(sh_proujin, body.m_xf);
end;


{ TMapObjectSpeeder }

constructor TMapObjectSpeeder.Create(world: Tb2World; pos, halfsize: Tb2vec2;
  angle: double; id: integer);
var
  bodyDef: Tb2BodyDef;
  shapeDef: Tb2PolygonDef;
begin
  bodyDef := Tb2BodyDef.Create;
  bodyDef.position := pos;
  body := world.CreateBody(bodyDef);

  shapeDef := Tb2PolygonDef.Create;
  shapeDef.vertexCount := 3;
  shapeDef.vertices[0] := b2vec2(0, halfsize.y);
  shapeDef.vertices[1] := b2vec2(-halfsize.x, -halfsize.y);
  shapeDef.vertices[2] := b2vec2(halfsize.x, -halfsize.y);
  shapeDef.isSensor := true;
  shape := body.CreateShape(shapeDef);
  body.SetXForm(body.GetPosition, DegToRad(angle-90));
  {shapeDef.SetAsBox(halfsize.x, halfsize.y, b2vec2(0, 0), DegToRad(angle));
  shapeDef.isSensor := true;
  shape := body.CreateShape(shapeDef);  }

  direction := angle;
  active := false;

  hs := halfsize;

  data.type_ := mot_Speeder;
  data.id := id;

  body.UserData := @data;
end;

procedure TMapObjectSpeeder.Draw;
begin
  if active then
  begin
    glColor3f(0.7, 0.5, 0.45);
    DrawPolygon(shape, body.m_xf);
  end;
end;

{ TMapObjectKicker }

constructor TMapObjectKicker.Create(world: Tb2World; x, y: double; hs: Tb2vec2;
  ang: double; id: integer);
var
  bodyDef: Tb2BodyDef;
  shapeDef: Tb2PolygonDef;
begin
  bodyDef := Tb2BodyDef.Create;
  bodyDef.position := b2vec2(x, y);
  body := world.CreateBody(bodyDef);

  shapeDef := Tb2PolygonDef.Create;
  shapeDef.SetAsBox(hs.x, hs.y, b2vec2(0, 0), DegToRad(ang));
  shapeDef.isSensor := true;
  shapeDef.restitution := IniMap.huini_restitution;
  shape := body.CreateShape(shapeDef);

  data.type_ := mot_Kick;
  data.id := id;
  data.dam := 5;

  body.UserData := @data;
end;

procedure TMapObjectKicker.Draw;
begin
  glColor3f(0.3, 0.3, 1);
  DrawPolygon(shape, body.m_xf);
end;

{ TScoresGroup }

procedure TScoresGroup.AddElement(t: TMapObjectType; id: integer);
begin
  SetLength(elements, length(elements)+1);
  elements[length(elements)-1].t := t;
  elements[length(elements)-1].id := id;
end;

function TScoresGroup.CountLikeCylinders;
var
  i, s, t, c: integer;
begin
  t := 0; c := 0;
  for i := 0 to length(elements) - 1 do
  begin
    if elements[i].t = mot_Trigger then
      if TMapObjectTrigger(map.objects[elements[i].id]).active then
        inc(t);
    if elements[i].t = mot_Cylinder then
      if TMapObjectCylinder(map.objects[elements[i].id]).color = TMapObjectCylinder(map.objects[id]).color then
        inc(c);
  end;
  s := 1 + t + c * (TMapObjectCylinder(map.objects[id]).color);
  Result := s;
end;

function TScoresGroup.CountLikeHoles: integer;
var
  i, s: integer;
begin
  s := 1;
  for i := 0 to length(elements) - 1 do
  begin
    if elements[i].t = mot_Trigger then
      if TMapObjectTrigger(map.objects[elements[i].id]).active then
        inc(s);
  end;
  Result := s;
end;

function TScoresGroup.CountLikeHuini: integer;
var
  i, s: integer;
begin
  s := 1;
  for i := 0 to length(elements) - 1 do
  begin
    if elements[i].t = mot_Trigger then
      if TMapObjectTrigger(map.objects[elements[i].id]).active then
        inc(s);
  end;
  Result := s;
end;

end.
